"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const validate_1 = require("@code-engine/validate");
const file_path_filter_1 = require("file-path-filter");
const nodeFS = require("fs");
const isGlob = require("is-glob");
const util_1 = require("util");
/**
 * Validates and normalizes the configuration.
 * @internal
 */
function normalizeConfig(config) {
    config = validate_1.validate.type.object(config, "config");
    let path = validate_1.validate.string.nonWhitespace(config.path, "path");
    let depth = validateDeep(config.deep);
    let [filter, filterCriteria] = validateFilter(config.filter);
    let fs = nodeFS;
    if (config.filter === undefined) {
        // Determine if the path is a glob pattern
        let glob;
        [path, glob] = splitGlob(path);
        if (glob) {
            // Only read files that match the glob pattern
            filterCriteria = glob;
            filter = file_path_filter_1.filePathFilter(glob);
        }
    }
    if (config.fs) {
        fs = {
            stat: validate_1.validate.type.function(config.fs.stat, "fs.stat", nodeFS.stat),
            lstat: validate_1.validate.type.function(config.fs.lstat, "fs.lstat", nodeFS.lstat),
            readdir: validate_1.validate.type.function(config.fs.readdir, "fs.readdir", nodeFS.readdir),
            readFile: validate_1.validate.type.function(config.fs.readFile, "fs.readFile", nodeFS.readFile),
            promises: {
                stat: util_1.promisify(validate_1.validate.type.function(config.fs.stat, "fs.stat", nodeFS.stat)),
                readFile: util_1.promisify(validate_1.validate.type.function(config.fs.readFile, "fs.readFile", nodeFS.readFile)),
            }
        };
    }
    return { path, depth, filter, filterCriteria, fs };
}
exports.normalizeConfig = normalizeConfig;
/**
 * Validates and normalizes the `deep` option.
 */
function validateDeep(deep) {
    validate_1.validate.type.oneOf(deep, [Boolean, Number, undefined], "deep option");
    switch (deep) {
        case true:
        case undefined:
            return Infinity;
        case false:
            return 0;
        default:
            return validate_1.validate.number.integer.nonNegative(deep, "deep option");
    }
}
/**
 * Validates and normalizes the `filter` option.
 */
function validateFilter(filter) {
    let filterCriteria;
    if (filter === undefined) {
        // There is no filter, so allow all files
        filterCriteria = undefined;
        filter = () => true;
    }
    else if (typeof filter === "string" || typeof filter === "boolean" || filter instanceof RegExp) {
        // This is a simple filter, so just return it as-is
        filterCriteria = filter;
        filter = file_path_filter_1.filePathFilter(filterCriteria);
    }
    else {
        // This is a more complicated filter, so wrap it in a filter function that accepts a CodeEngine File object.
        filter = filterCriteria = file_path_filter_1.createFilter({ map }, filter);
    }
    return [filter, filterCriteria];
}
/**
 * Splits a glob pattern into a directory and filter
 */
function splitGlob(path) {
    // Determine where the glob pattern starts
    let segments = path.split("/");
    let globSegment = segments.findIndex((segment) => isGlob(segment, { strict: false }));
    if (globSegment === -1) {
        // There is no glob pattern, it's just a path
        return [path, undefined];
    }
    else {
        // Split the directory path from the glob pattern
        return [
            segments.slice(0, globSegment).join("/"),
            segments.slice(globSegment).join("/")
        ];
    }
}
/**
 * Returns the path of the given file. Used for file filtering.
 */
function map(file) {
    return file.path;
}
//# sourceMappingURL=normalize-config.js.map